﻿//-----------------------------------------------------------------------
// <copyright file="Const.cs" company="tuliprules.org">
//     Copyright (c) 2011 tuplelister, tuliprules.org.
// </copyright>
// <summary>This is the Const class.</summary>
//-----------------------------------------------------------------------
namespace Tulip.Rules.Models.Rif.Core
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Xml.Serialization;

    /// <summary>
    /// Represents an individual, function, or predicate symbol, 
    /// with a 'type' attribute.
    /// </summary>
    [Serializable]
    [DesignerCategory(@"code")]
    [XmlType(AnonymousType = true, Namespace = "http://www.w3.org/2007/rif#")]
    [XmlRoot(Namespace = "http://www.w3.org/2007/rif#", IsNullable = false)]
    public class Const : Irimeta, IRifCoreElement
    {
        /// <summary>
        /// The text field.
        /// </summary>
        [EditorBrowsable(EditorBrowsableState.Never)]
        private List<string> textField;

        /// <summary>
        /// The type field.
        /// </summary>
        [EditorBrowsable(EditorBrowsableState.Never)]
        private string typeField;

        /// <summary>
        /// The lang field.
        /// </summary>
        [EditorBrowsable(EditorBrowsableState.Never)]
        private string langField;

        /// <summary>
        /// Initializes a new instance of the <see cref="Const"/> class. 
        /// </summary>
        public Const()
        {
            this.textField = new List<string>();
        }

        /// <summary>
        /// Gets or sets the text for mixed type.
        /// </summary>
        [XmlTextAttribute]
        public List<string> Text
        {
            get
            {
                return this.textField;
            }

            set
            {
                this.textField = value;
            }
        }

        /// <summary>
        /// Gets or sets the constant type.
        /// </summary>
        [XmlAttribute("type", DataType = "anyURI")]
        public string Type
        {
            get
            {
                return this.typeField;
            }

            set
            {
                this.typeField = value;
            }
        }

        /// <summary>
        /// Gets or sets the language of the constant.
        /// </summary>
        [XmlAttribute("lang", Form = System.Xml.Schema.XmlSchemaForm.Qualified, Namespace = "http://www.w3.org/XML/1998/namespace")]
        public string Lang
        {
            get
            {
                return this.langField;
            }

            set
            {
                this.langField = value;
            }
        }
    }
}
