﻿//-----------------------------------------------------------------------
// <copyright file="ContentGroundExprType.cs" company="tuliprules.org">
//     Copyright (c) 2011 tuplelister, tuliprules.org.
// </copyright>
// <summary>This is the ContentGroundExprType class.</summary>
//-----------------------------------------------------------------------

namespace Tulip.Rules.Models.Rif.Core
{
    using System;
    using System.ComponentModel;
    using System.Diagnostics.CodeAnalysis;
    using System.Xml.Serialization;

    /// <summary>
    /// Represents the content-GROUNDEXPR type.
    /// </summary>
    [Serializable]
    [DesignerCategory(@"code")]
    [XmlType(TypeName = "content-GROUNDEXPR.type", Namespace = "http://www.w3.org/2007/rif#")]
    public class ContentGroundExprType : Irimeta, IRifCoreElement
    {
        /// <summary>
        /// The op field.
        /// </summary>
        [SuppressMessage("StyleCop.CSharp.NamingRules", "SA1305:FieldNamesMustNotUseHungarianNotation",
            Justification = "Reviewed. Suppression is OK here."), EditorBrowsable(EditorBrowsableState.Never)]
        private Op opField;

        /// <summary>
        /// The args field.
        /// </summary>
        [EditorBrowsable(EditorBrowsableState.Never)] private ArgsGroundUnitermType argsField;

        /// <summary>
        /// Gets or sets the Atom/Expr role for predicates/functions as operations.
        /// </summary>
        [XmlElement("op", Order = 0)]
        public Op Op
        {
            get { return this.opField; }
            set { this.opField = value; }
        }

        /// <summary>
        /// Gets or sets the Atom/Expr positional arguments role, with ordered="yes" 
        /// attribute, containing n TERMs.
        /// </summary>
        [XmlElement("args", Order = 1)]
        public ArgsGroundUnitermType Args
        {
            get { return this.argsField; }
            set { this.argsField = value; }
        }
    }
}