﻿//-----------------------------------------------------------------------
// <copyright file="Import.cs" company="tuliprules.org">
//     Copyright (c) 2011 tuplelister, tuliprules.org.
// </copyright>
// <summary>This is the Import class.</summary>
//-----------------------------------------------------------------------
namespace Tulip.Rules.Models.Rif.Core
{
    using System;
    using System.ComponentModel;
    using System.Xml.Serialization;

    /// <summary>
    /// Represents an importation, containing location and optional profile.
    /// </summary>
    [Serializable]
    [DesignerCategory(@"code")]
    [XmlType(AnonymousType = true, Namespace = "http://www.w3.org/2007/rif#")]
    [XmlRoot(Namespace = "http://www.w3.org/2007/rif#", IsNullable = false)]
    public class Import : Irimeta, IRifCoreElement
    {
        /// <summary>
        /// The location field.
        /// </summary>
        [EditorBrowsable(EditorBrowsableState.Never)]
        private string locationField;

        /// <summary>
        /// The profile field.
        /// </summary>
        [EditorBrowsable(EditorBrowsableState.Never)]
        private string profileField;

        /// <summary>
        /// Gets or sets a location role, containing ANYURICONST.
        /// </summary>
        [XmlElement("location", DataType = "anyURI", Order = 0)]
        public string Location
        {
            get
            {
                return this.locationField;
            }

            set
            {
                this.locationField = value;
            }
        }

        /// <summary>
        /// Gets or sets a profile role, containing PROFILE,.
        /// </summary>
        [XmlElement("profile", DataType = "anyURI", Order = 1)]
        public string Profile
        {
            get
            {
                return this.profileField;
            }

            set
            {
                this.profileField = value;
            }
        }
    }
}
