﻿//-----------------------------------------------------------------------
// <copyright file="Items.cs" company="tuliprules.org">
//     Copyright (c) 2011 tuplelister, tuliprules.org.
// </copyright>
// <summary>This is the Items class.</summary>
//-----------------------------------------------------------------------
namespace Tulip.Rules.Models.Rif.Core
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Xml.Serialization;

    /// <summary>
    /// Represents a group of items (Const, List or content).
    /// </summary>
    [Serializable]
    [DesignerCategory(@"code")]
    [XmlType(AnonymousType = true, Namespace = "http://www.w3.org/2007/rif#")]
    [XmlRoot(Namespace = "http://www.w3.org/2007/rif#", IsNullable = false)]
    public class Items : IRifCoreElement
    {
        /// <summary>
        /// The items field.
        /// </summary>
        [EditorBrowsable(EditorBrowsableState.Never)]
        private List<object> itemsField;

        /// <summary>
        /// The ordered field.
        /// </summary>
        [EditorBrowsable(EditorBrowsableState.Never)]
        private string orderedField;

        /// <summary>
        /// Initializes a new instance of the <see cref="Items"/> class. 
        /// </summary>
        public Items()
        {
            this.itemsField = new List<object>();
            this.orderedField = "yes";
        }

        /// <summary>
        /// Gets or sets a list of items.  These can be Consts, Lists or content.
        /// </summary>
        [XmlElement("Const", typeof(Const), Order = 0)]
        [XmlElement("List", typeof(List), Order = 0)]
        [XmlElement("content", typeof(ContentGroundUnitermType), Order = 0)]
        public List<object> ItemList
        {
            get
            {
                return this.itemsField;
            }

            set
            {
                this.itemsField = value;
            }
        }

        /// <summary>
        /// Gets or sets a flag that indicates if the list of items is ordered.
        /// </summary>
        [XmlAttribute("ordered")]
        public string Ordered
        {
            get
            {
                return this.orderedField;
            }

            set
            {
                this.orderedField = value;
            }
        }
    }
}
