﻿// <copyright file="Forall.cs" company="tuliprules.org">
//     Copyright (c) 2011 tuplelister, tuliprules.org.
// </copyright>
// <summary>This is the Forall class.</summary>
//-----------------------------------------------------------------------
namespace Tulip.Rules.Models.Rif.Prd
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Xml.Serialization;

    /// <summary>
    /// Represents a quantified formula for 'Forall', containing declare and 
    /// formula roles.
    /// </summary>
    [Serializable]
    [DesignerCategory(@"code")]
    [XmlType(AnonymousType = true, Namespace = "http://www.w3.org/2007/rif#")]
    [XmlRoot(Namespace = "http://www.w3.org/2007/rif#", IsNullable = false)]
    public class Forall : Irimeta, IRifPrdElement
    {
        /// <summary>
        /// The declare field.
        /// </summary>
        [EditorBrowsable(EditorBrowsableState.Never)]
        private List<Declare> declarationsField;

        /// <summary>
        /// The pattern field.
        /// </summary>
        [EditorBrowsable(EditorBrowsableState.Never)]
        private List<Formula> patternsField;

        /// <summary>
        /// The formula field.
        /// </summary>
        [EditorBrowsable(EditorBrowsableState.Never)]
        private ForallFormula formulaField;

        /// <summary>
        /// Initializes a new instance of the <see cref="Forall"/> class. 
        /// </summary>
        public Forall()
        {
            this.declarationsField = new List<Declare>();
            this.patternsField = new List<Formula>();
        }

        /// <summary>
        /// Gets or sets a declare role, containing a Var.
        /// </summary>
        [XmlElement("declare", Order = 0)]
        public List<Declare> Declarations
        {
            get
            {
                return this.declarationsField;
            }

            set
            {
                this.declarationsField = value;
            }
        }

        /// <summary>
        /// Gets or sets a pattern role.
        /// </summary>
        [XmlElement("pattern", Order = 1)]
        public List<Formula> Patterns
        {
            get
            {
                return this.patternsField;
            }

            set
            {
                this.patternsField = value;
            }
        }

        /// <summary>
        /// Gets or sets a formula role, containing a FORMULA.
        /// </summary>
        [XmlElement("formula", Order = 2)]
        public ForallFormula Formula
        {
            get
            {
                return this.formulaField;
            }

            set
            {
                this.formulaField = value;
            }
        }
    }
}
