﻿//-----------------------------------------------------------------------
// <copyright file="Group.cs" company="tuliprules.org">
//     Copyright (c) 2011 tuplelister, tuliprules.org.
// </copyright>
// <summary>This is the Group class.</summary>
//-----------------------------------------------------------------------
namespace Tulip.Rules.Models.Rif.Prd
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Xml.Serialization;

    /// <summary>
    /// Represents a nested collection of sentences.
    /// </summary>
    [Serializable]
    [DesignerCategory(@"code")]
    [XmlType(AnonymousType = true, Namespace = "http://www.w3.org/2007/rif#")]
    [XmlRoot(Namespace = "http://www.w3.org/2007/rif#", IsNullable = false)]
    public class Group : Irimeta, IRifPrdElement
    {
        /// <summary>
        /// The behavior field.
        /// </summary>
        [EditorBrowsable(EditorBrowsableState.Never)]
        private Behavior behaviorField;

        /// <summary>
        /// The sentences list.
        /// </summary>
        [EditorBrowsable(EditorBrowsableState.Never)]
        private List<Sentence> sentencesField;

        /// <summary>
        /// Initializes a new instance of the <see cref="Group"/> class. 
        /// </summary>
        public Group()
        {
            this.sentencesField = new List<Sentence>();
        }

        /// <summary>
        /// Gets or sets behaviour of the rule engine.
        /// </summary>
        [XmlElement("behavior", Order = 0)]
        public Behavior Behavior
        {
            get
            {
                return this.behaviorField;
            }

            set
            {
                this.behaviorField = value;
            }
        }

        /// <summary>
        /// Gets or sets a list of sentence roles, containing RULE or Group.
        /// </summary>
        [XmlElement("sentence", Order = 1)]
        public List<Sentence> Sentences
        {
            get
            {
                return this.sentencesField;
            }

            set
            {
                this.sentencesField = value;
            }
        }
    }
}