﻿using System.Xml;
using System.Xml.Schema;

namespace Tulip.Rules.Tests
{
    using System.IO;
    using System.Xml.Serialization;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Models.Rif.Prd;

    [TestClass]
    public class TestRifPrd
    {
        [TestMethod]
        public void TestExampleFiles()
        {
            //this.TestExampleFile("Assert-premise");
            //this.TestExampleFile("AssertRetract-premise");
            //this.TestExampleFile("AssertRetract2-premise");
            //this.TestExampleFile("Conflict_resolution-premise");
            //this.TestExampleFile("Core_NonSafeness-input");
            //this.TestExampleFile("Core_NonSafeness_2-input");
            this.TestExampleFile("Fibonacci");
            this.TestExampleFile("Modify-premise");
            this.TestExampleFile("Modify_loop-premise");
            this.TestExampleFile("No_free_variables-input");
            this.TestExampleFile("Parents");
            this.TestExampleFile("Purchase");
            this.TestExampleFile("RDF_Combination_SubClass_5-premise");
            this.TestExampleFile("Retract-premise");
            this.TestExampleFile("Sort");
        }

        private void TestExampleFile(string fileName)
        {
            var path = @"..\..\..\Rules\Models\Rif\Prd\Samples\" + fileName + ".xml";
            var xsdPath = @"..\..\..\Rules\Models\Rif\Prd\Prd.xsd";

            var fs = new FileStream(xsdPath, FileMode.Open, FileAccess.Read);
            var schemas = new XmlSchemaSet();
            schemas.Add("http://www.w3.org/2007/rif#", XmlReader.Create(fs));

            var serializer = new XmlSerializer(typeof(Document));
            var reader = new StreamReader(path);
            var xrs = new XmlReaderSettings
            {
                DtdProcessing = DtdProcessing.Parse,
                ValidationType = ValidationType.Schema,
                Schemas = schemas
            };

            xrs.ValidationEventHandler += HandleValidationErrors;

            System.Diagnostics.Trace.WriteLine("Processing input file: " + path);

            var xr = XmlReader.Create(reader, xrs);
            var doc = (Document)serializer.Deserialize(xr);

            reader.Close();

            //var x = doc.ToString();

            // Serialize object to a text file.
            var pathOut = @"..\..\..\Rules\Models\Rif\Prd\Samples\" + fileName + "_OUT.xml";
            var xws = new XmlWriterSettings { Indent = true };
            var xmlWriter = XmlWriter.Create(pathOut, xws);
            serializer.Serialize(xmlWriter, doc);
            xmlWriter.Close();

            // Validate the output.
            fs = new FileStream(xsdPath, FileMode.Open, FileAccess.Read);
            reader = new StreamReader(path);
            xr = XmlReader.Create(reader, xrs);
            while (xr.Read()) { }
        }

        private void HandleValidationErrors(object sender, ValidationEventArgs args)
        {
            System.Diagnostics.Trace.WriteLine(args.Message + ": " + args.Exception.LineNumber + ", " + args.Exception.LinePosition);
        }
    }
}
