﻿//-----------------------------------------------------------------------
// <copyright file="TestRuleSet.cs" company="tuliprules.org">
//     Copyright (c) 2010 tuplelister, tuliprules.org.
// </copyright>
// <summary>This is the TestRuleSet class.</summary>
//-----------------------------------------------------------------------

namespace Tulip.Rules.Tests
{
    using System.IO;
    using System.Xml;
    using System.Xml.Serialization;
    using Microsoft.VisualStudio.TestTools.UnitTesting;

    [TestClass]
    public class TestRuleSet
    {
        /// <summary>
        /// Tests serialization and deserialization of the RuleSet class.
        /// </summary>
        [TestMethod]
        public void TestRuleSetSerialization()
        {
            // Test serialization of an individual RuleSet
            var parser = new RuleSetParser();
            parser.Parse("This is a dummy ruleset.");

            var ruleSet1 = new RuleSet
            {
                Name = "Ruleset 1",
                MajorVersion = 1,
                MinorVersion = 0,
                Build = 0,
                Revision = 0,
                Parser = parser
            };

            var serializer = new XmlSerializer(typeof(RuleSet));

            using (var ms = new MemoryStream())
            {
                serializer.Serialize(ms, ruleSet1);
                var xmlDocument = new XmlDocument();
                ms.Seek(0, SeekOrigin.Begin);
                xmlDocument.Load(ms);

                // Deserialize back to RuleSet
                using (var msOut = new MemoryStream())
                {
                    xmlDocument.Save(msOut);
                    msOut.Seek(0, SeekOrigin.Begin);
                    var ruleSetOut = (RuleSet)serializer.Deserialize(msOut);

                    Assert.AreEqual(ruleSet1.Name, ruleSetOut.Name, "The deserialized rule set does not have the same name as the original.");
                    Assert.AreEqual(ruleSet1.MajorVersion, ruleSetOut.MajorVersion, "The deserialized rule set does not have the same major version as the original.");
                    Assert.AreEqual(ruleSet1.MinorVersion, ruleSetOut.MinorVersion, "The deserialized rule set does not have the same minor version as the original.");
                    Assert.AreEqual(ruleSet1.Build, ruleSetOut.Build, "The deserialized rule set does not have the same build as the original.");
                    Assert.AreEqual(ruleSet1.Revision, ruleSetOut.Revision, "The deserialized rule set does not have the same revision as the original.");
                    Assert.AreEqual(ruleSet1.Presentation, ruleSetOut.Presentation, "The deserialized rule set does not have the same content as the original.");
                }
            }
        }
    }
}
